#ifndef DATABASE_H_
#define DATABASE_H_

/**
 * \file database.h
 * \brief Header file for different modules. Generally covers
 * database routines.
 * 
 * This header contains declarations of functions, structures
 * and constants related to TINE database file access.
 * This includes csv file access as well as address resolution
 * and caching routines.
 * 
 * Functions found here are implemented in
 * <ol>
 * <li>dblib.c</li>
 * <li>srvdbase.c</li>
 * <li>others (not yet completely analyzed)</li>
 * </ol>
 * 
 * \todo list all modules for which this is a header.
 */

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#define TXT 0
#define INT 1
#define FLT 2
#define HEX 3
#define SHT 4
#define FMT 5
#define ACC 6

/**
 * \internal
 * \brief A structure which contains information about a column in a CSV file.
 * 
 * This structure is used internally by CSVDBStruct.
 */
typedef struct
{
  char *nam;
  char *def;
  int  pos;
  int  fmt;
  int  siz;
  BYTE *ptr;
  int (*fcn)(char *);
  BYTE *start;
} CSVCOLUMN;

/**
 * \internal
 * \brief Container for the contents of a CSV file.
 * 
 * This structure has two purposes. It serves as
 * <ul>
 * <li>a container</li>
 * <li>its own single iterator</li>
 * </ul>
 * 
 * \see CSVCOLUMN
 * \todo misleading field names siz (=nRows), tgt_siz (=rowSize), cnt (=currentRow)
 * \todo redevelopment of csv library desirable
 */
typedef struct CSVDBStruct
{
  CSVCOLUMN *col;                    /* pointer to CSVCOLUMN list */
  int ncols;                         /* num columns */
  int siz;                           /* size of db file excluding comments => num rows - header */
  void (*ini)(struct CSVDBStruct *); /* optional initializer routine */
  int  (*itr)(struct CSVDBStruct *); /* optional iterator */
  void (*exi)(struct CSVDBStruct *); /* optional finalizer routine */
  int cnt;                           /* counter */
  BYTE *tgt;                         /* pointer to the container */
  int tgt_siz;                       /* size of container (e.g. sizeof(some typedef) */
} CSVDB;

typedef struct    /* local */
{
  char UserName[16];
} UserListStruct;

#if !defined(UNIX) && !defined(VXWORKS)
char *strtok_r(char *s, const char *delim, char **ptrptr);
#endif
char *strtrm(char *s);
int csvAssignValue(void *ptr,int fmt,int siz,char *str);
int csvReadFile(char *path,char *file,CSVDB *db,void **target);
int csvWriteFile(char *path,char *file,CSVDB *db);
void csvStdInit(struct CSVDBStruct *db);
int  csvStdIterator(struct CSVDBStruct *db);
void csvStdExit(struct CSVDBStruct *db);
int GetDataBaseSize(char *filename);
int locateFecName(char *fecname);
int locateTagName(char *expname,char *context,int *rpcidx);
int asyncLocateFecName(char *fecname);
int asyncLocateTagName(char *expname,char *context);
int GetAddressFromNameFile(char *context,char *expname, char *fecname,int *rpcidx);
int GetAddressFromNameServer(char *context,char *expname, char *fecname,int *rpcidx);
int GetAllocSizesFromNameServer(int *RPCsize,int *FECsize);
int GetAllocSizesFromNameFile(int *RPCsize,int *FECsize);
int appendAddressToCache(char *fecname,char *ip, short portOffset);
int appendServerToCache(char *fecname,char *context,char *expname, char *eqpname);
int namcmp(char *name,char *str,int col,int limit);
int findcol(char *str,char *tag);
int getColumnStr(int col,char *str,char *buf,int bufsiz);
char *colptr(int col,char *str);
void BytesToHex(BYTE *b,char *s,int n);
void HexToBytes(char *s,BYTE *b,int nibs);
int GetNameServerAddress(void);
int getFecName(void);
int appendRegisteredUserList(char *user);
int removeRegisteredUser(char *user);
int getRegisteredUsers(UserListStruct **ul,char *fn,int *nr,char *accstr,int *accflg);
void dumpRegisteredUsers(void);
int getRegisteredExports(void);
char *GetExportedName(char *eqpname);
NAME32 *GetEqpModuleNames(char *eqname,int num);
char *GetDeviceName(char *eqm,int devnr);
int GetDeviceNumber(char *eqm,char *dev);
int AssignNumModules(char *eqname,int num);
int GetPropertyId(char *eqm,char *prpName);
int outputConsumerList(void);
int outputContractList(void);
int outputContractStats(int i);
int outputConnectionStats(int k);
int outputServerStats(void);
int outputServerTime(void);
int dumpContractStats(int i);
int dumpConnectionStats(int i);
int dumpServerStats(void);
int dumpServerTime(void);
int dumpVersion(void);

extern FILE *dbgfp;
extern char dbgbuf[256];
extern char tagNameFilter[64];
extern int dbaseloaded;
extern int MaxRPCAllocation;
extern char ControlDBpath[80];
extern char gRPCDataBaseFullName[128];
extern char gFECDataBaseFullName[128];
extern int WRITEaccess;
extern int DENYaccess;
extern int NrOfUsers;
extern int NrOfDeniedUsers;
extern char FecDBpath[80];
extern int FecNameRegistered;
extern int ExportsRegistered;
extern int gotNameServerAddress;
extern UserListStruct *UserList;
extern UserListStruct *DeniedUserList;
extern int isNameServerRequest;
extern int NameServerLoaded;

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* DATABASE_H_ */
